// Learn cc.Class:
//  - https://docs.cocos.com/creator/manual/en/scripting/class.html
// Learn Attribute:
//  - https://docs.cocos.com/creator/manual/en/scripting/reference/attributes.html
// Learn life-cycle callbacks:
//  - https://docs.cocos.com/creator/manual/en/scripting/life-cycle-callbacks.html

const defaultData = require('../script/defaultData');

cc.Class({
    extends: cc.Component,

    properties: {
        starPreFab: {
            type: cc.Prefab,
            default: null,
        },
        sharkPreFab: {
            type: cc.Prefab,
            default: null,
        },
        arrowPrefab: {
            type: cc.Prefab,
            default: null,
        },
        bigAPrefab: {
            type: cc.Prefab,
            default: null,
        },
        bigBPrefab: {
            type: cc.Prefab,
            default: null,
        },
    },

    // LIFE-CYCLE CALLBACKS:

    // onLoad () {},

    ctor() {
        this._imageResList = [];
        this._audioResList = [];
    },

    start() {
        this.data = defaultData;
        this.getData((data) => {
            if (data) {
                this.data = data;
            }
            this.preloadItem();
        });
    },

    getData(func) {
        if (window && window.courseware) {
            window.courseware.getData(func, 'Z_05_nxuan1');
        } else {
            func(defaultData);
        }
    },

    _imageResList: null,
    _audioResList: null,
    preloadItem() {
        this.addPreloadImage();
        this.addPreloadAudio();
        this.preload();
        this.onLoadFinish();
    },

    addPreloadImage() { },

    addPreloadAudio() {
    },

    preload() {
        const preloadArr = this._imageResList.concat(this._audioResList);
        cc.assetManager.loadAny(preloadArr, null, null, (err, data) => {
            this.hideAirClassLoading();
        });
        if (preloadArr.length == 0) {
            this.hideAirClassLoading();
        }
    },

    hideAirClassLoading() {
        if (window && window["air"]) {
            window["air"].hideAirClassLoading();
            console.log('hideAirClassLoading');
        }
    },

    initListeners() {

    },

    onLoadFinish() {
        this.initListeners();

        this.createLetterBig('B');
    },

    createLetterBig(letter) {
        const letterNodeBase = cc.instantiate(this[`big${letter}Prefab`]);
        letterNodeBase.name = 'letterNodeBase';
        letterNodeBase.parent = this.node;

        let currentSelectIdx = 2;
        const btnUpdate = letterNodeBase.getChildByName('BtnUpdate');
        btnUpdate.on('click', () => {
            currentSelectIdx++;
            if (currentSelectIdx >= 3) {
                currentSelectIdx = 0;
            }
            this.updateArrows(letterNodeBase, currentSelectIdx);
            console.log('currentSelectIdx = ' + currentSelectIdx);
        });
        this.updateArrows(letterNodeBase, currentSelectIdx);
    },

    updateArrows(letterNodeBase, idx) {
        const nodePosBase = letterNodeBase.getChildByName('NodePosBase');

        if (this.arrowPrefabList) {
            this.arrowPrefabList.forEach(node => {
                node.parent = null;
            });
        }
        this.arrowPrefabList = [];

        for (let i = 0; i < 5; i++) {
            let startNode = nodePosBase.getChildByName(`NodePos_${i}_0`);
            if (!startNode) {
                break;
            }

            startNode.opacity = 0;
            let lastNode = startNode;
            for (let j = 1; j < 100; j++) {
                let endNode = nodePosBase.getChildByName(`NodePos_${i}_${j}`);
                if (!endNode) {
                    break;
                }
                endNode.opacity = 0;
                const lineVec2 = cc.v2(endNode.x - lastNode.x, endNode.y - lastNode.y);
                const lineLength = lineVec2.mag();

                const arrowNumber = Math.ceil(lineLength / 100);

                for (let k = 0; k < arrowNumber; k++) {
                    if (i != idx) {
                        continue;
                    }
                    const arrowPrefab = cc.instantiate(this.arrowPrefab);
                    arrowPrefab.x = lastNode.x + lineVec2.x * (k + 1) / (arrowNumber + 1);
                    arrowPrefab.y = lastNode.y + lineVec2.y * (k + 1) / (arrowNumber + 1);
                    arrowPrefab.angle = cc.v2(1, 0).signAngle(lineVec2) * 180 / Math.PI;
                    arrowPrefab.name = name;
                    arrowPrefab.parent = this.node;
                    this.arrowPrefabList.push(arrowPrefab);

                    if (k == 0 && j == 1) {
                        console.log('arrowPrefab.angle = ' + arrowPrefab.angle);
                        this.createShark(startNode, arrowPrefab.angle);
                    }

                    const nodeArrow = arrowPrefab.getChildByName('NodeArrow');
                    cc.tween(nodeArrow)
                        .delay((j + k) * 0.3)
                        .to(0.2, { opacity: 0 })
                        .delay(0.5)
                        .to(0.2, { opacity: 255 })
                        .delay(7 - (j + k) * 0.3)
                        .union()
                        .repeatForever()
                        .start();
                }
                lastNode = endNode;
            }
            if (i == idx) {
                this.createStar(lastNode);
            }
        }
    },

    createShark(startNode, angle) {
        if (this.sharkNode) {
            this.sharkNode.parent = null;
            this.sharkNode = null;
        }
        const sharkNode = cc.instantiate(this.sharkPreFab);
        sharkNode.parent = this.node;
        sharkNode.x = startNode.x;
        sharkNode.y = startNode.y;
        sharkNode.angle = angle;
        this.sharkNode = sharkNode;
    },

    createStar(endNode) {
        if (this.starNode) {
            this.starNode.parent = null;
            this.starNode = null;
        }
        const starNode = cc.instantiate(this.starPreFab);
        starNode.parent = this.node;
        starNode.x = endNode.x;
        starNode.y = endNode.y;
        this.starNode = starNode;
    },

    addDragListeners(node) {
        if (node.dragListeners) {
            return;
        }
        node.dragListeners = true;
        node.on('touchstart', (event) => {
            const location = event.getLocation();
            node.startLocation = location;
            node.startPos = node.getPosition();
        });
        node.on('touchmove', (event) => {
            if (!node.startLocation) {
                return;
            }
            const location = event.getLocation();
            node.x = node.startPos.x + (location.x - node.startLocation.x);
            node.y = node.startPos.y + (location.y - node.startLocation.y);
        });
        node.on('touchend', (event) => {
            node.startLocation = null;
            node.startPos = null;
            this.updateArrows(this.node.getChildByName('letterNodeBase'));
        });
    },

    // update (dt) {},
});
